@@ -1,5 +1,7 @@
 #include "arcjobcontroller.h"
 
+#include "arc-gui-config.h"
+
 #include <arc/ArcConfig.h>
 #include <arc/ArcLocation.h>
 #include <arc/DateTime.h>
@@ -11,16 +13,26 @@
 #include <arc/UserConfig.h>
 #include <arc/Utils.h>
 #include <arc/XMLNode.h>
+#if ARC_VERSION_MAJOR >= 3
+#include <arc/compute/Broker.h>
+#include <arc/compute/ComputingServiceRetriever.h>
+#include <arc/compute/ExecutionTarget.h>
+#include <arc/compute/Job.h>
+#include <arc/compute/JobDescription.h>
+#include <arc/compute/SubmissionStatus.h>
+#include <arc/compute/Submitter.h>
+#include <arc/compute/JobSupervisor.h>
+#include <arc/compute/JobInformationStorage.h>
+#else
+#include <arc/client/ComputingServiceRetriever.h>
+#include <arc/client/Submitter.h>
+#include <arc/client/JobSupervisor.h>
 #include <arc/client/Broker.h>
 #include <arc/client/JobDescription.h>
-#ifndef ARC_VERSION_2
-#include <arc/client/TargetGenerator.h>
 #endif
-#include <arc/client/Submitter.h>
 #include <arc/loader/Plugin.h>
 #include <arc/loader/FinderLoader.h>
 #include <arc/credential/Credential.h>
-#include <arc/client/JobSupervisor.h>
 
 #include <QStringList>
 #include <QTableWidget>
@@ -35,6 +47,7 @@
 #include <QPainter>
 #include <QLabel>
 #include <QSettings>
+#include <QProcess>
 
 #include "qdebugstream.h"
 
@@ -266,29 +279,28 @@ void ArcJobController::newJobList(const QString &jobListName)
 {
     if (m_jobListTable!=0)
     {
-        /*
-        //QTableWidget* jobTable = new QTableWidget(0, 3);
-        m_jobTable->clear();
-        m_jobTable->setRowCount(0);
-        m_jobTable->setColumnCount(3);
-        QStringList labels;
-        labels << "JobID" << "Name" << "State";
-        m_jobTable->setHorizontalHeaderLabels(labels);
-        m_jobTable->horizontalHeader()->setResizeMode(0, QHeaderView::Stretch);
-        m_jobTable->verticalHeader()->hide();
-        m_jobTable->setShowGrid(true);
-        m_jobTable->setFrameStyle(QFrame::NoFrame);
-        m_jobTable->setSelectionBehavior(QTableWidget::SelectRows);
-        m_jobTable->setSelectionMode(QAbstractItemView::MultiSelection);
-        */
+        bool jobListExists = false;
 
-        JmJobList* jmJobList = new JmJobList();
-        jmJobList->setFilename(jobListName);
-        m_jmJobLists.append(jmJobList);
+        qDebug() << "Checking existing job lists.";
 
-        // Store index to m_jobList in table, to handle case when table is sorted.
+        for (int i=0; i<m_jmJobLists.count(); i++)
+        {
+            qDebug() << m_jmJobLists[i]->filename();
+            if (m_jmJobLists[i]->filename() == jobListName)
+                jobListExists = true;
+        }
 
-        m_currentJmJobList = jmJobList;
+        if (!jobListExists)
+        {
+            qDebug() << "Adding job list";
+            JmJobList* jmJobList = new JmJobList();
+            jmJobList->setFilename(jobListName);
+            m_jmJobLists.append(jmJobList);
+
+            // Store index to m_jobList in table, to handle case when table is sorted.
+
+            m_currentJmJobList = jmJobList;
+        }
 
         this->updateJobList();
         this->updateJobTable();
@@ -324,12 +336,16 @@ void ArcJobController::setDownloadDir(const QString& downloadDir)
     m_downloadDir = downloadDir;
 }
 
+#if ARC_VERSION_MAJOR >= 3
 void ArcJobController::queryJobStatus(JmJobList* jobList)
 {
-#ifndef ARC_VERSION_2
     // Read existing jobs from job list file
 
-    Arc::Job::ReadAllJobsFromFile(jobList->filename().toStdString(), m_arcJobList);
+    //Arc::Job::ReadJobIDsFromFile(jobList->filename().toStdString(), m_arcJobList);
+    //Arc::Job::ReadAllJobsFromFile(jobList->filename().toStdString(), m_arcJobList);
+
+    Arc::JobInformationStorageXML jobStorage(jobList->filename().toStdString());
+    jobStorage.ReadAll(m_arcJobList);
 
     // Clear the current JmJobList;
 
@@ -340,40 +356,31 @@ void ArcJobController::queryJobStatus(JmJobList* jobList)
     std::list<Arc::Job>::iterator jli;
 
     for (jli=m_arcJobList.begin(); jli!=m_arcJobList.end(); jli++) {
-        QString jobId = (*jli).JobID.fullstr().c_str();
+        QString jobId = (*jli).JobID.c_str();
         QString jobName = (*jli).Name.c_str();
         jobList->add(jobId, jobName, "Unknown");
     }
 
     // Create job supervisor and job controllers
 
-    if (m_jobSupervisor!=0)
-        delete m_jobSupervisor;
+    Arc::JobSupervisor jobSupervisor(m_userConfig, m_arcJobList);
 
-    m_jobSupervisor = new Arc::JobSupervisor(m_userConfig, m_arcJobList);
-    std::list<Arc::JobController*> jobControllers = m_jobSupervisor->GetJobControllers();
-
-    // Query status
-
-    std::list<std::string> status;
-    std::vector<const Arc::Job*> jobs;
-    std::list<Arc::JobController*>::iterator cit;
-    for (cit=jobControllers.begin(); cit!=jobControllers.end(); cit++) {
-        //cout << "Querying job controller." << endl;
-        (*cit)->FetchJobs(status, jobs);
-    }
+    jobSupervisor.Update();
+    m_arcJobList = jobSupervisor.GetSelectedJobs();
 
     // Update JmJobList
 
-    std::vector<const Arc::Job*>::iterator sit;
+    std::list<Arc::Job>::iterator sit;
 
-    for (sit=jobs.begin(); sit!=jobs.end(); sit++) {
-        QString jobId = (*sit)->JobID.fullstr().c_str();
-        QString jobState = (*sit)->State.GetGeneralState().c_str();
+    for (sit=m_arcJobList.begin(); sit!=m_arcJobList.end(); sit++) {
+        QString jobId = (*sit).JobID.c_str();
+        QString jobState = (*sit).State.GetGeneralState().c_str();
         jobList->fromJobId(jobId)->setState(jobState);
     }
+}
 #else
-
+void ArcJobController::queryJobStatus(JmJobList* jobList)
+{
     // Read existing jobs from job list file
 
     Arc::Job::ReadAllJobsFromFile(jobList->filename().toStdString(), m_arcJobList);
@@ -417,8 +424,8 @@ void ArcJobController::queryJobStatus(JmJobList* jobList)
         QString jobState = (*sit).State.GetGeneralState().c_str();
         jobList->fromJobId(jobId)->setState(jobState);
     }
-#endif
 }
+#endif
 
 void ArcJobController::queryJobStatus()
 {
@@ -531,6 +538,24 @@ void ArcJobController::clearSelection()
     m_jobTable->clearSelection();
 }
 
+void ArcJobController::openSessionDir()
+{
+    // Return if nothing is selected
+
+    if (m_selectedJobIds.size()==0)
+        return;
+
+    if (m_selectedJobIds.size()!=1)
+        return;
+
+    QProcess process;
+    std::list<std::string>::iterator jli = m_selectedJobIds.begin();
+    QString commandLine = (*jli).c_str();
+    commandLine = "arcstorage-ui " + commandLine;
+    process.startDetached(commandLine);
+}
+
+#if ARC_VERSION_MAJOR >= 3
 void ArcJobController::cleanJobs()
 {
     // Return if nothing is selected
@@ -538,23 +563,71 @@ void ArcJobController::cleanJobs()
     if (m_selectedJobIds.size()==0)
         return;
 
-#ifndef ARC_VERSION_2
+    // Read existing jobs from job list file
+
+    //Arc::Job::ReadAllJobsFromFile(m_currentJmJobList->filename().toStdString(), m_arcJobList);
+
+    Arc::JobInformationStorageXML jobStorage(m_currentJmJobList->filename().toStdString());
+    jobStorage.ReadAll(m_arcJobList);
 
     // Create job supervisor and job controllers
 
-    qDebug() << "Creating job supervisor...";
-    Arc::JobSupervisor jobSupervisor(m_userConfig, m_selectedJobIds);
-    qDebug() << "Getting job controllers...";
-    std::list<Arc::JobController*> jobControllers = jobSupervisor.GetJobControllers();
-    std::list<Arc::JobController*>::iterator cit;
+    std::cout << "Creating job supervisor..." << std::endl;
+    Arc::JobSupervisor jobSupervisor(m_userConfig, m_arcJobList);
+    jobSupervisor.Update();
 
-    std::list<std::string> status;
+    std::list<std::string>::iterator sli;
+    std::list<std::string> jobIds;
 
-    for (cit=jobControllers.begin(); cit!=jobControllers.end(); cit++) {
-        qDebug() << "Cleaning job...";
-        (*cit)->Clean(status, false);
+    for (sli=m_selectedJobIds.begin(); sli!=m_selectedJobIds.end(); sli++)
+        jobIds.push_back(*sli);
+
+    std::cout << "jobId len = " << jobIds.size() << std::endl;
+
+    std::cout << "Job ids to clean -->" << std::endl;
+    for (sli=jobIds.begin(); sli!=jobIds.end(); sli++)
+        std::cout << (*sli) << std::endl;
+    std::cout << "<--" << std::endl;
+
+    jobSupervisor.ClearSelection();
+    jobSupervisor.SelectByID(jobIds);
+
+    if (jobSupervisor.GetSelectedJobs().empty())
+        std::cout << "No jobs selected." << std::endl;
+
+    if (jobSupervisor.Clean())
+    {
+        std::cout << "Succesfully cleaned job(s)." << std::endl;
     }
+    else
+    {
+        std::cout << "Failed cleaning job(s)." << std::endl;
+    }
+
+    std::list<std::string> cleaned = jobSupervisor.GetIDsProcessed();
+    const std::list<std::string>& notcleaned = jobSupervisor.GetIDsNotProcessed();
+
+    //for (std::list<Arc::Job>::const_iterator it = jobSupervisor.GetAllJobs().begin(); it != jobSupervisor.GetAllJobs().end(); ++it) {
+    //   if (it->State == Arc::JobState::UNDEFINED) {
+    //     cleaned.push_back(it->JobID);
+    //   }
+    // }
+
+    std::cout << "Jobs to be cleaned = " << cleaned.size() << std::endl;
+    std::cout << "Jobs to not cleaned = " << notcleaned.size() << std::endl;
+
+    jobStorage.Remove(cleaned);
+
+    if (jobStorage.Remove(cleaned))
+        std::cout << "Succesfully removed jobs from " << m_currentJmJobList->filename().toStdString() << std::endl;
+}
 #else
+void ArcJobController::cleanJobs()
+{
+    // Return if nothing is selected
+
+    if (m_selectedJobIds.size()==0)
+        return;
 
     // Read existing jobs from job list file
 
@@ -609,9 +682,10 @@ void ArcJobController::cleanJobs()
 
     if (Arc::Job::RemoveJobsFromFile(m_currentJmJobList->filename().toStdString(), cleaned))
         std::cout << "Succesfully removed jobs from " << m_currentJmJobList->filename().toStdString() << std::endl;
-#endif
 }
+#endif
 
+#if ARC_VERSION_MAJOR >= 3
 void ArcJobController::resubmitJobs()
 {
     // Return if nothing is selected
@@ -637,7 +711,119 @@ void ArcJobController::resubmitJobs()
     std::string debug;
     std::string broker;
 
-#ifndef ARC_VERSION_2
+    if (m_jobSupervisor!=0)
+        delete m_jobSupervisor;
+
+    // Read existing jobs from job list file
+
+    Arc::JobInformationStorageXML jobStorage(m_currentJmJobList->filename().toStdString());
+    jobStorage.ReadAll(m_arcJobList);
+
+    m_jobSupervisor = new Arc::JobSupervisor(m_userConfig, m_arcJobList);
+    m_jobSupervisor->Update();
+    m_jobSupervisor->ClearSelection();
+    m_jobSupervisor->SelectByID(m_selectedJobIds);
+
+    if (!m_jobSupervisor->GetSelectedJobs().size() == 0)
+    {
+      std::cout << Arc::IString("No jobs") << std::endl;
+      return;
+    }
+
+    std::list<Arc::Endpoint> endPoints;
+    std::list<Arc::Job> resubmittedJobs;
+
+    m_jobSupervisor->Resubmit(2, endPoints, resubmittedJobs);
+
+    // Clearing jobs.
+
+    m_selectedJobIds.clear();
+}
+#else
+void ArcJobController::resubmitJobs()
+{
+    // Return if nothing is selected
+
+    if (m_selectedJobIds.size()==0)
+        return;
+
+    //-------------------------------------------------
+
+    bool all = false;
+    std::string joblist;
+    std::string jobidfileout;
+    std::list<std::string> jobidfilesin;
+    std::list<std::string> clusters;
+    std::list<std::string> qlusters;
+    std::list<std::string> indexurls;
+    bool keep = false;
+    bool same = false;
+    std::list<std::string> status;
+    bool show_plugins = false;
+    int timeout = -1;
+    std::string conffile;
+    std::string debug;
+    std::string broker;
+
+    if (m_jobSupervisor!=0)
+        delete m_jobSupervisor;
+
+    // Read existing jobs from job list file
+
+    Arc::Job::ReadAllJobsFromFile(m_currentJmJobList->filename().toStdString(), m_arcJobList);
+
+    m_jobSupervisor = new Arc::JobSupervisor(m_userConfig, m_arcJobList);
+    m_jobSupervisor->Update();
+    m_jobSupervisor->ClearSelection();
+
+    std::list<Arc::URL> jobIds;
+    std::list<Arc::URL>::iterator uit;
+    std::list<std::string>::iterator sit;
+
+    for (sit=m_selectedJobIds.begin(); sit!=m_selectedJobIds.end(); sit++)
+        jobIds.push_back(Arc::URL(*sit));
+
+    m_jobSupervisor->SelectByID(jobIds);
+
+    if (!m_jobSupervisor->GetSelectedJobs().size() == 0)
+    {
+      std::cout << Arc::IString("No jobs") << std::endl;
+      return;
+    }
+
+    std::list<Arc::Endpoint> endPoints;
+    std::list<Arc::Job> resubmittedJobs;
+
+    m_jobSupervisor->Resubmit(2, endPoints, resubmittedJobs);
+
+    // Clearing jobs.
+
+    m_selectedJobIds.clear();
+
+    /*
+
+    // Return if nothing is selected
+
+    if (m_selectedJobIds.size()==0)
+        return;
+
+    //-------------------------------------------------
+
+    bool all = false;
+    std::string joblist;
+    std::string jobidfileout;
+    std::list<std::string> jobidfilesin;
+    std::list<std::string> clusters;
+    std::list<std::string> qlusters;
+    std::list<std::string> indexurls;
+    bool keep = false;
+    bool same = false;
+    std::list<std::string> status;
+    bool show_plugins = false;
+    int timeout = -1;
+    std::string conffile;
+    std::string debug;
+    std::string broker;
 
     // Different selected services are needed in two different context,
     // so the two copies of UserConfig objects will contain different
@@ -789,11 +975,11 @@ void ArcJobController::resubmitJobs()
           qDebug() << "Job could not be killed or cleaned";
       }
     }
-#else
-
-#endif
+    */
 }
+#endif
 
+#if ARC_VERSION_MAJOR >= 3
 void ArcJobController::killJobs()
 {
     // Return if nothing is selected
@@ -801,23 +987,48 @@ void ArcJobController::killJobs()
     if (m_selectedJobIds.size()==0)
         return;
 
-#ifndef ARC_VERSION_2
+    // Read existing jobs from job list file
+
+    Arc::JobInformationStorageXML jobStorage(m_currentJmJobList->filename().toStdString());
+    jobStorage.ReadAll(m_arcJobList);
 
     // Create job supervisor and job controllers
 
-    qDebug() << "Creating job supervisor...";
-    Arc::JobSupervisor jobSupervisor(m_userConfig, m_selectedJobIds);
-    qDebug() << "Getting job controllers...";
-    std::list<Arc::JobController*> jobControllers = jobSupervisor.GetJobControllers();
-    std::list<Arc::JobController*>::iterator cit;
+    std::cout << "Creating job supervisor..." << std::endl;
+    Arc::JobSupervisor jobSupervisor(m_userConfig, m_arcJobList);
+    jobSupervisor.Update();
+    jobSupervisor.ClearSelection();
+    jobSupervisor.SelectByID(m_selectedJobIds);
 
-    std::list<std::string> status;
+    if (jobSupervisor.GetSelectedJobs().empty())
+        std::cout << "No jobs selected." << std::endl;
 
-    for (cit=jobControllers.begin(); cit!=jobControllers.end(); cit++) {
-        qDebug() << "Kill job...";
-        (*cit)->Kill(status, false);
+    if (jobSupervisor.Cancel())
+    {
+        std::cout << "Succesfully cancelled job(s)." << std::endl;
     }
+    else
+    {
+        std::cout << "Failed cancelling job(s)." << std::endl;
+    }
+
+    std::list<std::string> cleaned = jobSupervisor.GetIDsProcessed();
+    const std::list<std::string>& notcleaned = jobSupervisor.GetIDsNotProcessed();
+
+    std::cout << "Jobs to be cancelled = " << cleaned.size() << std::endl;
+    std::cout << "Jobs to not cancelled = " << notcleaned.size() << std::endl;
+
+    if (jobStorage.Remove(cleaned))
+        std::cout << "Succesfully removed jobs from " << m_currentJmJobList->filename().toStdString() << std::endl;
+}
 #else
+void ArcJobController::killJobs()
+{
+    // Return if nothing is selected
+
+    if (m_selectedJobIds.size()==0)
+        return;
+
     // Read existing jobs from job list file
 
     Arc::Job::ReadAllJobsFromFile(m_currentJmJobList->filename().toStdString(), m_arcJobList);
@@ -871,10 +1082,10 @@ void ArcJobController::killJobs()
 
     if (Arc::Job::RemoveJobsFromFile(m_currentJmJobList->filename().toStdString(), cleaned))
         std::cout << "Succesfully removed jobs from " << m_currentJmJobList->filename().toStdString() << std::endl;
-
-#endif
 }
+#endif
 
+#if ARC_VERSION_MAJOR >= 3
 void ArcJobController::getJobs()
 {
     // Return if nothing is selected
@@ -887,26 +1098,149 @@ void ArcJobController::getJobs()
     if (m_downloadDir == "")
         return;
 
-#ifndef ARC_VERSION_2
+    Arc::JobSupervisor jobSupervisor(m_userConfig, m_arcJobList);
 
-    // Create job supervisor and job controllers
+    jobSupervisor.Update();
+    jobSupervisor.SelectValid();
+    jobSupervisor.ClearSelection();
+    jobSupervisor.SelectByID(m_selectedJobIds);
 
-    qDebug() << "Creating job supervisor...";
-    Arc::JobSupervisor jobSupervisor(m_userConfig, m_selectedJobIds);
-    qDebug() << "Getting job controllers...";
-    std::list<Arc::JobController*> jobControllers = jobSupervisor.GetJobControllers();
-    std::list<Arc::JobController*>::iterator cit;
-
-    std::list<std::string> status;
-
-    for (cit=jobControllers.begin(); cit!=jobControllers.end(); cit++) {
-        qDebug() << "Get job...";
-        (*cit)->Get(status, m_downloadDir.toStdString(), false, false);
+    /*
+    if (!opt.status.empty()) {
+      jobmaster.SelectByStatus(opt.status);
     }
-#else
+    */
 
-#endif
+    if (jobSupervisor.GetSelectedJobs().empty()) {
+      std::cout << Arc::IString("No jobs") << std::endl;
+      return;
+    }
+
+    std::list<std::string> downloaddirectories;
+    int retval = (int)!jobSupervisor.Retrieve(m_downloadDir.toStdString(), true, false, downloaddirectories);
+
+    for (std::list<std::string>::const_iterator it = downloaddirectories.begin();
+         it != downloaddirectories.end(); ++it) {
+      std::cout << Arc::IString("Results stored at: %s", *it) << std::endl;
+    }
+
+    unsigned int retrieved_num = jobSupervisor.GetIDsProcessed().size();
+    unsigned int notretrieved_num = jobSupervisor.GetIDsNotProcessed().size();
+    unsigned int cleaned_num = 0;
+
+    bool keepFiles= false;
+
+    if (!keepFiles) {
+      std::list<std::string> retrieved = jobSupervisor.GetIDsProcessed();
+      // No need to clean selection because retrieved is subset of selected
+      jobSupervisor.SelectByID(retrieved);
+      if(!jobSupervisor.Clean()) {
+        std::cout << Arc::IString("Warning: Some jobs were not removed from server") << std::endl;
+        std::cout << Arc::IString("         Use arclean to remove retrieved jobs from job list", m_userConfig.JobListFile()) << std::endl;
+        retval = 1;
+      }
+      cleaned_num = jobSupervisor.GetIDsProcessed().size();
+
+      Arc::JobInformationStorageXML jobStorage(m_currentJmJobList->filename().toStdString());
+
+      if (!jobStorage.Remove(jobSupervisor.GetIDsProcessed())) {
+        std::cout << Arc::IString("Warning: Failed to lock job list file %s", m_userConfig.JobListFile()) << std::endl;
+        std::cout << Arc::IString("         Use arclean to remove retrieved jobs from job list", m_userConfig.JobListFile()) << std::endl;
+        retval = 1;
+      }
+
+      std::cout << Arc::IString("Jobs processed: %d, successfully retrieved: %d, successfully cleaned: %d", retrieved_num+notretrieved_num, retrieved_num, cleaned_num) << std::endl;
+
+    } else {
+
+      std::cout << Arc::IString("Jobs processed: %d, successfully retrieved: %d", retrieved_num+notretrieved_num, retrieved_num) << std::endl;
+
+    }
 }
+#else
+void ArcJobController::getJobs()
+{
+    // Return if nothing is selected
+
+    if (m_selectedJobIds.size()==0)
+        return;
+
+    // Do we have a download dir?
+
+    if (m_downloadDir == "")
+        return;
+
+    Arc::JobSupervisor jobSupervisor(m_userConfig, m_arcJobList);
+
+    jobSupervisor.Update();
+    jobSupervisor.SelectValid();
+
+    std::list<std::string>::iterator sli;
+    std::list<Arc::URL> jobIds;
+    std::list<Arc::URL>::iterator jii;
+
+    for (sli=m_selectedJobIds.begin(); sli!=m_selectedJobIds.end(); sli++)
+        jobIds.push_back(Arc::URL(*sli));
+
+    std::cout << "Job ids to download -->" << std::endl;
+    for (jii=jobIds.begin(); jii!=jobIds.end(); jii++)
+        std::cout << (*jii) << std::endl;
+    std::cout << "<--" << std::endl;
+
+    jobSupervisor.ClearSelection();
+    jobSupervisor.SelectByID(jobIds);
+
+    /*
+    if (!opt.status.empty()) {
+      jobmaster.SelectByStatus(opt.status);
+    }
+    */
+
+    if (jobSupervisor.GetSelectedJobs().empty()) {
+      std::cout << Arc::IString("No jobs") << std::endl;
+      return;
+    }
+
+    std::list<std::string> downloaddirectories;
+    int retval = (int)!jobSupervisor.Retrieve(m_downloadDir.toStdString(), true, false, downloaddirectories);
+
+    for (std::list<std::string>::const_iterator it = downloaddirectories.begin();
+         it != downloaddirectories.end(); ++it) {
+      std::cout << Arc::IString("Results stored at: %s", *it) << std::endl;
+    }
+
+    unsigned int retrieved_num = jobSupervisor.GetIDsProcessed().size();
+    unsigned int notretrieved_num = jobSupervisor.GetIDsNotProcessed().size();
+    unsigned int cleaned_num = 0;
+
+    bool keepFiles= false;
+
+    if (!keepFiles) {
+      std::list<Arc::URL> retrieved = jobSupervisor.GetIDsProcessed();
+      // No need to clean selection because retrieved is subset of selected
+      jobSupervisor.SelectByID(retrieved);
+      if(!jobSupervisor.Clean()) {
+        std::cout << Arc::IString("Warning: Some jobs were not removed from server") << std::endl;
+        std::cout << Arc::IString("         Use arclean to remove retrieved jobs from job list", m_userConfig.JobListFile()) << std::endl;
+        retval = 1;
+      }
+      cleaned_num = jobSupervisor.GetIDsProcessed().size();
+
+      if (!Arc::Job::RemoveJobsFromFile(m_userConfig.JobListFile(), jobSupervisor.GetIDsProcessed())) {
+        std::cout << Arc::IString("Warning: Failed to lock job list file %s", m_userConfig.JobListFile()) << std::endl;
+        std::cout << Arc::IString("         Use arclean to remove retrieved jobs from job list", m_userConfig.JobListFile()) << std::endl;
+        retval = 1;
+      }
+
+      std::cout << Arc::IString("Jobs processed: %d, successfully retrieved: %d, successfully cleaned: %d", retrieved_num+notretrieved_num, retrieved_num, cleaned_num) << std::endl;
+
+    } else {
+
+      std::cout << Arc::IString("Jobs processed: %d, successfully retrieved: %d", retrieved_num+notretrieved_num, retrieved_num) << std::endl;
+
+    }
+}
+#endif
 
 void ArcJobController::jobTableSelectionChanged()
 {   
