@@ -12,6 +12,7 @@
 #include <QFileInfo>
 #include <QDateTime>
 #include <QDesktopServices>
+#include <QMenuItem>
 
 #include <iostream>
 #include "arcstoragewindow.h"
@@ -63,12 +64,6 @@ ArcStorageWindow::ArcStorageWindow(QWidget *parent, bool childWindow, QString Ur
     if (!m_childWindow)
         m_windowId = -1;
 
-    // Load settings from disk
-
-    //Settings::loadFromDisk();
-    //QVariant qvar = Settings::getValue("urlList");
-    //QList<QVariant> urlList = qvar.toList();
-
     // Initialise state variables
 
     m_folderWidgetBeingUpdated = NULL;
@@ -88,16 +83,16 @@ ArcStorageWindow::ArcStorageWindow(QWidget *parent, bool childWindow, QString Ur
 
     // Create and add the url combobox manually to the toolbar because QT Designer doesn't support it
 
-    m_urlComboBox.setEditable(true);
-    m_urlComboBox.setMaxVisibleItems(10);
-    m_urlComboBox.setMaxCount(10);
-    //for (int i = 0; i < urlList.size(); ++i)
-    //    m_urlComboBox.addItem(urlList.at(i).toString());
-    m_urlComboBox.setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
-    m_urlComboBox.repaint();
-    ui->mainToolBar->addWidget(&m_urlComboBox);
-    connect(m_urlComboBox.lineEdit(), SIGNAL(returnPressed()), this, SLOT(onURLEditReturnPressed()));  // When someone presses return in the url combobox...
-    connect(&m_urlComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(onUrlComboBoxCurrentIndexChanged(int)));
+    m_urlCompleter = new QCompleter();
+    m_urlEdit.setCompleter(m_urlCompleter);
+    m_urlCompleteButton.setArrowType(Qt::DownArrow);
+    m_urlCompleteButton.setMaximumHeight(28);
+    ui->mainToolBar->addWidget(&m_urlEdit);
+    ui->mainToolBar->addWidget(&m_urlCompleteButton);
+
+    connect(&m_urlEdit, SIGNAL(returnPressed()), this, SLOT(onURLEditReturnPressed()));  // When someone presses return in the url combobox...
+    connect(&m_urlCompleteButton, SIGNAL(clicked()), this, SLOT(onUrlCompletePressed()));
+    connect(m_urlCompleter, SIGNAL(activated(QString)), this, SLOT(onUrlCompleteActivated()));
 
     // Can't add empty space in toolbar, so we add a dummy widget instead.
 
@@ -129,7 +124,6 @@ ArcStorageWindow::ArcStorageWindow(QWidget *parent, bool childWindow, QString Ur
     ui->filesTreeWidget->setColumnCount(fileTreeHeaderLabels.size());
     ui->filesTreeWidget->setHeaderLabels(fileTreeHeaderLabels);
     ui->filesTreeWidget->setSelectionMode(QAbstractItemView::ExtendedSelection);
-
     ui->filesTreeWidget->setEditTriggers(QTreeWidget::SelectedClicked | QTreeWidget::EditKeyPressed);
 
     setBusyUI(true);
@@ -214,6 +208,8 @@ ArcStorageWindow::ArcStorageWindow(QWidget *parent, bool childWindow, QString Ur
         m_fileProcessingThread = new FileTransferProcessingThread();
         m_fileProcessingThread->start();
     }
+
+    this->updateBookmarkMenu();
 }
 
 ArcStorageWindow::~ArcStorageWindow()
@@ -363,15 +359,6 @@ void ArcStorageWindow::pushUrl(QString url)
     {
         m_backStack.push_back(url);
         ui->actionBack->setEnabled(true);
-
-        QVectorIterator<QString> i(m_backStack);
-
-        /*
-        qDebug() << "-->";
-        while (i.hasNext())
-            qDebug() << i.next();
-        qDebug() << "<--";
-        */
     }
 }
 
@@ -404,7 +391,13 @@ void ArcStorageWindow::showEvent(QShowEvent *e)
 
     // Update file list
 
-    m_currentFileServer->updateFileList(m_startUrl);
+    if (m_startUrl.length()!=0)
+        m_currentFileServer->updateFileList(m_startUrl);
+    else
+    {
+        m_startUrl = QDir::homePath();
+        m_currentFileServer->updateFileList(m_startUrl);
+    }
 
     setCurrentComboBoxURL(m_currentFileServer->getCurrentURL());
 
@@ -426,17 +419,6 @@ void ArcStorageWindow::showEvent(QShowEvent *e)
 void ArcStorageWindow::closeEvent(QCloseEvent *e)
 {
     logger.msg(Arc::VERBOSE, "Received close event. (closeEvent)");
-    // This method is called before window is closed.
-    // Saves settings. Closes child windows.
-    /*
-    QList<QVariant> urlList;
-
-    for (int i = 0; i < m_urlComboBox.count(); ++i)
-        urlList << m_urlComboBox.itemText(i);
-
-    Settings::setValue("urlList", urlList);
-    Settings::saveToDisk();
-    */
 
     if (!m_childWindow)
         this->writeSettings();
@@ -518,7 +500,7 @@ void ArcStorageWindow::setBusyUI(bool busy)
         ui->menuBar->setEnabled(false);
         ui->mainToolBar->setEnabled(false);
         ui->filesTreeWidget->setEnabled(false);
-        m_urlComboBox.setEnabled(false);
+        m_urlEdit.setEnabled(false);
         ui->foldersTreeWidget->setEnabled(false);
         ui->urlBreadCrumbToolbar->setEnabled(false);
     }
@@ -529,7 +511,7 @@ void ArcStorageWindow::setBusyUI(bool busy)
         ui->menuBar->setEnabled(true);
         ui->mainToolBar->setEnabled(true);
         ui->filesTreeWidget->setEnabled(true);
-        m_urlComboBox.setEnabled(true);
+        m_urlEdit.setEnabled(true);
         ui->foldersTreeWidget->setEnabled(true);
         ui->urlBreadCrumbToolbar->setEnabled(true);
     }
@@ -663,16 +645,13 @@ void ArcStorageWindow::updateFileTree()
         item->setText(0, AFE->getFileName());
         if (AFE->getFileType()==ARCDir)
         {
-            //item->setIcon(0,QIcon::fromTheme("folder"));
             item->setIcon(0,QIcon(":/resources/icons/16px/Folder Open.png"));
             item->setText(1, "---");
             item->setFlags(Qt::ItemIsEditable | Qt::ItemIsEnabled | Qt::ItemIsSelectable);
         }
         else
         {
-            //item->setIcon(0,QIcon::fromTheme("document"));
             item->setIcon(0,QIcon(":/resources/icons/16px/Untitled.png"));
-            //item->setText(1, QString::number(AFE->getSize()));
             item->setText(1, convertToSizeWithUnit(AFE->getSize()));
             item->setFlags(Qt::ItemIsEditable | Qt::ItemIsEnabled | Qt::ItemIsSelectable);
         }
@@ -692,8 +671,27 @@ void ArcStorageWindow::updateFileTree()
     for (int i=0; i<8; i++)
         ui->filesTreeWidget->resizeColumnToContents(i);
 
+    // Update recent file list
+
     this->setCurrentComboBoxURL(m_currentFileServer->getCurrentURL());
 
+    m_recent.insert(m_currentFileServer->getCurrentURL());
+
+    // Create auto completer from updated recent urls
+
+    QStringList urls;
+
+    if (m_urlCompleter!=0)
+        delete m_urlCompleter;
+
+    QSetIterator<QString> i(m_recent);
+    while (i.hasNext())
+        urls.append(i.next());
+
+    m_urlCompleter = new QCompleter(urls);
+    connect(m_urlCompleter, SIGNAL(activated(QString)), this, SLOT(onUrlCompleteActivated(QString)));
+    m_urlEdit.setCompleter(m_urlCompleter);
+
     logger.msg(Arc::VERBOSE, "File list update done.");
 }
 
@@ -760,6 +758,21 @@ void ArcStorageWindow::updateFoldersTreeBelow()
     logger.msg(Arc::INFO, "Folder tree update done.");
 }
 
+void ArcStorageWindow::updateBookmarkMenu()
+{
+    for (int i=0; i<m_bookmarkActions.length(); i++)
+        ui->menuBookmarks->removeAction(m_bookmarkActions.at(i));
+
+    for (int i=0; i<GlobalStateInfo::instance()->bookmarkCount(); i++)
+    {
+        QAction* action = new QAction(ui->menuBookmarks);
+        m_bookmarkActions.append(action);
+        action->setText(GlobalStateInfo::instance()->bookmark(i));
+        ui->menuBookmarks->addAction(action);
+        connect(action, SIGNAL(triggered()), this, SLOT(onBookmarkTriggered()));
+    }
+}
+
 void ArcStorageWindow::expandFolderTreeWidget(QTreeWidgetItem *folderWidget)
 {
     logger.msg(Arc::VERBOSE, "Expanding folder in tree. (expandFolderTreeWidget)");
@@ -788,59 +801,49 @@ void ArcStorageWindow::expandFolderTreeWidget(QTreeWidgetItem *folderWidget)
 QString ArcStorageWindow::getCurrentComboBoxURL()
 {
     QString url = "";
-
-    QLineEdit *le = m_urlComboBox.lineEdit();
-    if (le != NULL)
-    {
-        url = le->text();
-    }
-
+    url = m_urlEdit.text();
     return url;
 }
 
 void ArcStorageWindow::setCurrentComboBoxURL(QString url)
 {
-    QLineEdit *le = m_urlComboBox.lineEdit();
-    if (le != NULL)
+    ui->urlBreadCrumbToolbar->clear();
+    m_breadCrumbItems.clear();
+
+    QStringList urlParts = url.split("/");
+    QString buildPath = "";
+
+    if (urlParts.at(0).length()==0)
     {
-        ui->urlBreadCrumbToolbar->clear();
-        m_breadCrumbItems.clear();
+        buildPath = "";
 
-        QStringList urlParts = url.split("/");
-        QString buildPath = "";
-
-        if (urlParts.at(0).length()==0)
+        for (int i=1; i<urlParts.count(); i++)
         {
-            buildPath = "";
-
-            for (int i=1; i<urlParts.count(); i++)
-            {
-                QAction* action = ui->urlBreadCrumbToolbar->addAction(urlParts.at(i));
-                ui->urlBreadCrumbToolbar->addSeparator();
-                buildPath += "/" + urlParts.at(i);
-                m_breadCrumbItems.append(buildPath);
-                action->setStatusTip(buildPath);
-                action->setToolTip(buildPath);
-                connect(action, SIGNAL(triggered()), this, SLOT(onBreadCrumbTriggered()));
-            }
+            QAction* action = ui->urlBreadCrumbToolbar->addAction(urlParts.at(i));
+            ui->urlBreadCrumbToolbar->addSeparator();
+            buildPath += "/" + urlParts.at(i);
+            m_breadCrumbItems.append(buildPath);
+            action->setStatusTip(buildPath);
+            action->setToolTip(buildPath);
+            connect(action, SIGNAL(triggered()), this, SLOT(onBreadCrumbTriggered()));
         }
-        else
-        {
-            buildPath = urlParts.at(0)+"/";
-
-            for (int i=2; i<urlParts.count(); i++)
-            {
-                QAction* action = ui->urlBreadCrumbToolbar->addAction(urlParts.at(i));
-                ui->urlBreadCrumbToolbar->addSeparator();
-                buildPath += "/" + urlParts.at(i);
-                m_breadCrumbItems.append(buildPath);
-                action->setStatusTip(buildPath);
-                action->setToolTip(buildPath);
-                connect(action, SIGNAL(triggered()), this, SLOT(onBreadCrumbTriggered()));
-            }
-        }
-        le->setText(url);
     }
+    else
+    {
+        buildPath = urlParts.at(0)+"/";
+
+        for (int i=2; i<urlParts.count(); i++)
+        {
+            QAction* action = ui->urlBreadCrumbToolbar->addAction(urlParts.at(i));
+            ui->urlBreadCrumbToolbar->addSeparator();
+            buildPath += "/" + urlParts.at(i);
+            m_breadCrumbItems.append(buildPath);
+            action->setStatusTip(buildPath);
+            action->setToolTip(buildPath);
+            connect(action, SIGNAL(triggered()), this, SLOT(onBreadCrumbTriggered()));
+        }
+    }
+    m_urlEdit.setText(url);
 }
 
 void ArcStorageWindow::onFilesDroppedInFileListWidget(QList<QUrl>& urlList)
@@ -857,6 +860,7 @@ void ArcStorageWindow::onFilesDroppedInFileListWidget(QList<QUrl>& urlList)
         logger.msg(Arc::VERBOSE, "Dropped file :" + urlList[i].toString().toStdString());
 
     FileTransferList::instance()->pauseProcessing();
+    FileTransferList::instance()->startMeasuring();
     m_currentFileServer->copyToServer(urlList, m_currentFileServer->getCurrentPath());
     FileTransferList::instance()->resumeProcessing();
 }
@@ -871,7 +875,15 @@ void ArcStorageWindow::onFileListFinished(bool error, QString errorMsg)
     {
         logger.msg(Arc::ERROR, "Update of file list failed.");
         m_currentUpdateFileListsMode = CUFLM_noUpdate;
-        QMessageBox::information(this, tr("ArcFTP"), errorMsg);
+        QMessageBox::information(this, tr("ARC Storage Explorer"), errorMsg);
+        QString url;
+        if (this->m_backStack.size()==0)
+            url = QDir::homePath();
+        else
+            url = this->popUrl();
+
+        this->setCurrentComboBoxURL(url);
+        this->openUrl(url);
     }
     else
     {
@@ -918,12 +930,14 @@ void ArcStorageWindow::onFileListFinished(bool error, QString errorMsg)
 
 void ArcStorageWindow::onCopyFromServerFinished(bool error)
 {
+    FileTransferList::instance()->stopMeasuring();
+
     logger.msg(Arc::VERBOSE, "Copy from file server finished. (onCopyFromServerFinished)");
     updateFileTree();
     GlobalStateInfo::instance()->hideTransferWindow();
     setBusyUI(false);
     if (error == true)
-        QMessageBox::information(this, tr("ArcFTP"), "An error occured while trying to copy the file");
+        QMessageBox::information(this, tr("ARC Storage Explorer"), "An error occured while trying to copy the file");
 
     if (m_filesToOpen.length()>0)
     {
@@ -946,7 +960,7 @@ void ArcStorageWindow::onDeleteFinished(bool error)
 
     setBusyUI(false);
     if (error == true)
-        QMessageBox::information(this, tr("ArcFTP"), "Delete failed");
+        QMessageBox::information(this, tr("ARC Storage Explorer"), "Could not delete file(s)");
 }
 
 
@@ -963,6 +977,8 @@ void ArcStorageWindow::onMakeDirFinished(bool error)
 
 void ArcStorageWindow::onCopyToServerFinished(bool error, QList<QString> &failedFiles)
 {
+    FileTransferList::instance()->stopMeasuring();
+
     logger.msg(Arc::VERBOSE, "Copy to server finished. (onCopyToServerFinished)");
     m_currentUpdateFileListsMode = CUFLM_clickedFolder;   // Update the listview displaying the folder...
     onFileListFinished(false, "");                      // ... so that the deleted file is removed
@@ -1117,16 +1133,39 @@ void ArcStorageWindow::onURLEditReturnPressed()
     on_actionReload_triggered();
 }
 
+void ArcStorageWindow::onUrlCompletePressed()
+{
+    m_urlEdit.completer()->widget()->setFocus();
+    m_urlEdit.completer()->complete();
+}
+
+void ArcStorageWindow::onUrlCompleteActivated(const QString& text)
+{
+    on_actionReload_triggered();
+}
+
 void ArcStorageWindow::onUrlComboBoxCurrentIndexChanged(int index)
 {
 }
 
+void ArcStorageWindow::onEditTextChanged(const QString& text)
+{
+
+}
+
 void ArcStorageWindow::onBreadCrumbTriggered()
 {
     QAction* action = (QAction*)sender();
     this->openUrl(action->toolTip());
 }
 
+void ArcStorageWindow::onBookmarkTriggered()
+{
+    QAction* action = (QAction*)sender();
+    this->openUrl(action->text());
+}
+
+
 void ArcStorageWindow::on_foldersTreeWidget_expanded(QModelIndex index)
 {
     logger.msg(Arc::VERBOSE, "on_foldersTreeWidget_expanded() %d ", index.row());
@@ -1221,7 +1260,7 @@ void ArcStorageWindow::on_filesTreeWidget_itemDoubleClicked(QTreeWidgetItem *ite
         m_folderWidgetBeingUpdated = item;
         this->pushUrl(m_currentFileServer->getCurrentURL());
         m_currentFileServer->startUpdateFileList(newURL);
-        setCurrentComboBoxURL(m_currentFileServer->getCurrentURL());
+        //setCurrentComboBoxURL(m_currentFileServer->getCurrentURL());
     }
     else
     {
@@ -1288,6 +1327,10 @@ void ArcStorageWindow::openUrl(QString url)
     setBusyUI(true);
     m_currentUpdateFileListsMode = CUFLM_syncBoth;
 
+    // Remember last path
+
+    this->pushUrl(m_currentFileServer->getCurrentURL());
+
     // Delete previous file server
 
     if (m_currentFileServer!=0)
@@ -1312,10 +1355,8 @@ void ArcStorageWindow::openUrl(QString url)
 
     while (url.endsWith('/')) { url = url.left(url.length() - 1); }  // Get rid of trailing /
 
-    this->pushUrl(m_currentFileServer->getCurrentURL());
     m_folderListUrl = url;
     m_currentFileServer->startUpdateFileList(url);
-    setCurrentComboBoxURL(m_currentFileServer->getCurrentURL());
 }
 
 void ArcStorageWindow::on_actionReload_triggered()
@@ -1734,3 +1775,35 @@ void ArcStorageWindow::on_actionOpenURLExt_triggered()
         FileTransferList::instance()->resumeProcessing();
     }
 }
+
+void ArcStorageWindow::on_actionAddBookmark_triggered()
+{
+    if (m_urlEdit.text().length()!=0)
+    {
+        GlobalStateInfo::instance()->addBookmark(m_urlEdit.text());
+    }
+}
+
+void ArcStorageWindow::on_actionEditBookmarks_triggered()
+{
+    QStringList items;
+
+    for (int i=0; i<GlobalStateInfo::instance()->bookmarkCount(); i++)
+        items << GlobalStateInfo::instance()->bookmark(i);
+
+    bool ok;
+    QString item = QInputDialog::getItem(this, tr("Delete bookmark"),
+                                         tr("Bookmark"), items, 0, false, &ok);
+    if (ok && !item.isEmpty())
+    {
+        GlobalStateInfo::instance()->removeBookmark(item);
+    }
+}
+
+void ArcStorageWindow::on_actionClearBookmarks_triggered()
+{
+    if (m_urlEdit.text().length()!=0)
+    {
+        GlobalStateInfo::instance()->clearBookmarks();
+    }
+}
