@@ -24,6 +24,14 @@
 #include <QObject>
 #include <QString>
 
+#if QT_VERSION < QT_VERSION_CHECK(4, 8, 0)
+#include <QTime>
+typedef QTime QElapsedTimer;
+#else
+#include <QTime>
+#include <QElapsedTimer>
+#endif
+
 /// File transfer states
 enum TTransferState { TS_IDLE,     ///<Waiting to be transferred.
                       TS_EXECUTED, ///<Transfer has been initiated.
@@ -61,6 +69,9 @@ private:
     Arc::UserConfig* m_config;
     Arc::DataStatus m_status;
     Arc::SimpleCondition m_cond;
+
+    QTime m_transferTimer;
+    qint64 m_transferTime;
 public:
     /// Create a file transfer object.
     /**
@@ -122,6 +133,10 @@ public:
     /// Return current transfer statistics.
     void getTransferStatus(unsigned long& transferred, unsigned long& totalSize);
 
+    double transferTime();
+    unsigned long totalTransferred();
+    unsigned long totalSize();
+
 Q_SIGNALS:
 
     /// Signal for updating transfer status
