@@ -6,6 +6,13 @@
 #include <QHash>
 #include <QThread>
 
+#if QT_VERSION < QT_VERSION_CHECK(4, 8, 0)
+#include <QTime>
+typedef QTime QElapsedTimer;
+#else
+#include <QElapsedTimer>
+#endif
+
 #include "filetransfer.h"
 
 /// Thread class for driving the FileTransferList.
@@ -44,6 +51,9 @@ private:
     FileTransferProcessingThread* m_fileProcessingThread;
     QMutex m_accessMutex;
     int m_maxTransfers;
+    QElapsedTimer m_transferTimer;
+    qint64 m_transferTime;
+    unsigned long m_totalTransferred;
 public:
     /// Returns the global FileTransferList instance.
     static FileTransferList* instance()
@@ -136,6 +146,15 @@ public:
      */
     void resumeProcessing();
 
+    void startMeasuring();
+    void stopMeasuring();
+
+    unsigned long totalTransferred();
+    double transferTime();
+
+    int maxTransfers();
+    void setMaxTransfers(int maxTransfers);
+
 private:
     /// Create a file transfer list
     FileTransferList();
@@ -156,6 +175,10 @@ Q_SIGNALS:
 
     /// This signal is sent when a file transfer is removed from the list.
     void onRemoveTransfer(QString id);
+
+private Q_SLOTS:
+    void onCompleted(FileTransfer* fileTransfer, bool success, QString error);
+
 };
 
 #endif // FILETRANSFERLIST_H
