@@ -26,7 +26,7 @@ ARCTools::ARCTools()
     m_helpWindow = 0;
 }
 
-bool ARCTools::initUserConfig()
+bool ARCTools::initUserConfig(bool showUi)
 {
     m_proxyController->initialize();
 
@@ -72,6 +72,8 @@ bool ARCTools::initUserConfig()
         m_proxyController->showProxyUI();
     else if (proxyStatus == ArcProxyController::PS_NOT_VALID)
         m_proxyController->showProxyUI();
+    else if (showUi)
+        m_proxyController->showProxyUI();
 
     if (m_proxyController->getUiReturnStatus()==ArcProxyController::RS_FAILED)
         prereqFound = false;
