@@ -67,6 +67,14 @@ ProxyWindow::ProxyWindow(QWidget *parent, ArcProxyController* proxyController) :
 
     m_configTableDirty = false;
 
+    ui->NSSProfileList->clear();
+    for (i=0; i<m_proxyController->nssPathCount(); i++)
+    {
+        ui->NSSProfileList->addItem(m_proxyController->getNssPath(i));
+    }
+
+    ui->NSSProfileList->setCurrentRow(0);
+
     this->readSettings();
 }
 
@@ -136,18 +144,34 @@ void ProxyWindow::on_generateButton_clicked()
 {   
     // Ask for private key passphrase
 
-    bool ok;
-    QString passphrase = QInputDialog::getText(this, tr("Proxy generation"),
-                                               tr("Private key passphrase:"), QLineEdit::Password,
-                                               "", &ok);
+    QString passphrase;
 
-    if (!ok)
-        return;
+    if (ui->tabWidget->currentIndex()==0)
+        m_proxyController->setUseNssDb(true);
+    else
+        m_proxyController->setUseNssDb(false);
 
-    if (passphrase.length()==0)
+    if (m_proxyController->useNssDb())
     {
-        QMessageBox::warning(this, "Proxy generation", "Passphrase can't be empy.");
-        return;
+        if (ui->NSSProfileList->currentRow()>=0)
+            m_proxyController->setNssPath(ui->NSSProfileList->currentItem()->text());
+    }
+
+    if (!m_proxyController->useNssDb())
+    {
+        bool ok;
+        passphrase = QInputDialog::getText(this, tr("Proxy generation"),
+                                                   tr("Private key passphrase:"), QLineEdit::Password,
+                                                   "", &ok);
+
+        if (!ok)
+            return;
+
+        if (passphrase.length()==0)
+        {
+            QMessageBox::warning(this, "Proxy generation", "Passphrase can't be empy.");
+            return;
+        }
     }
 
     // Generate proxy using ArcProxyController
@@ -254,3 +278,33 @@ void ProxyWindow::on_helpButton_clicked()
     m_helpWindow->setWindowFlags(m_helpWindow->windowFlags() | Qt::WindowStaysOnTopHint);
     m_helpWindow->show();
 }
+
+void ProxyWindow::on_NSSProfileList_itemClicked(QListWidgetItem *item)
+{
+    QString selectedPath = item->text();
+    m_proxyController->setNssPath(selectedPath);
+}
+
+void ProxyWindow::on_tabWidget_currentChanged(int index)
+{
+    if (ui->tabWidget->currentIndex()==0)
+        m_proxyController->setUseNssDb(true);
+    else
+        m_proxyController->setUseNssDb(false);
+
+    m_proxyController->initialize();
+
+    ui->identityText->setText(m_proxyController->getIdentity());
+
+    QDateTime currentTime = QDateTime::currentDateTime();
+    QDateTime notValidAfter = currentTime.addSecs(12*60*60);
+
+    ui->proxyLifeTimeEdit->setDateTime(notValidAfter);
+    ui->generateButton->setFocus();
+
+    if (m_proxyController->getUseGSIProxy())
+        ui->proxyTypeCombo->setCurrentIndex(1);
+    else
+        ui->proxyTypeCombo->setCurrentIndex(0);
+
+}
